
Baggins_Outfitter = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0","AceEvent-2.0")

local Baggins_Outfitter = Baggins_Outfitter

local dewdrop = AceLibrary:HasInstance("Dewdrop-2.0") and AceLibrary("Dewdrop-2.0")
Baggins_Outfitter.sets_table = {}
Baggins_Outfitter.items_table = {}

Baggins_Outfitter.itemLinkRegex = "|Hitem:(-?%d+):(-?%d+):(-?%d+):"

local function Matches(bag, slot, rule)
	local items_table = Baggins_Outfitter.items_table
	
	local setName = rule.set
	if not setName then return false end
	
	local itemLink = GetContainerItemLink(bag, slot)
	if not itemLink then return false end
	
	local startIndex, endIndex, strItemID = itemLink:find(Baggins_Outfitter.itemLinkRegex)
	local itemID = tonumber(strItemID)
	
	if items_table[itemID] then
		for index, itemOutfitName in ipairs(items_table[itemID]) do
			if (setName == itemOutfitName) or (rule.any == true) then
				return true
			end
		end
	end
	
	return false
	
end

local function GetName(rule)
	local setName = rule.set or "???"

	if rule.any then
		setName = "Any"
	end
	
	return "Outfitter: " .. setName
end

local function CleanRule(rule)
	rule.set = ""
	rule.any = true
	Baggins:OnRuleChanged()
end

local function setAnySet(rule, bool)
	rule.any = bool
	rule.set = ""
	Baggins:OnRuleChanged()
end

local function setRuleSet(rule, set)
	rule.set = set
	rule.any = false
	Baggins:OnRuleChanged()
end

local function DewDropOptions(rule, level, value)
	local sets_table = Baggins_Outfitter.sets_table

	if level == 1 then
		dewdrop:AddLine("text", "Outfitter", "isTitle", true)
		dewdrop:AddLine("text", "Any", "checked", rule.any == true, 'func', setAnySet, 'arg1', rule, 'arg2', true)
	
		for key, subtable in pairs(Baggins_Outfitter.sets_table) do
			dewdrop:AddLine("text", "Outfitter: " .. key, "isTitle", true)
			
			for subkey, value in pairs(subtable) do
				dewdrop:AddLine("text", value["Name"], "checked", rule.set == value["Name"], 'func', setRuleSet, 'arg1', rule, 'arg2', value["Name"])
			end
		end
	end
end

function Baggins_Outfitter:CacheSets()
	Baggins_Outfitter.sets_table = {}
	Baggins_Outfitter.items_table = {}
	
	local sets_table = Baggins_Outfitter.sets_table
	local items_table = Baggins_Outfitter.items_table

	for categoryKey, category in pairs(gOutfitter_Settings.Outfits) do
		Baggins_Outfitter.sets_table[categoryKey] = gOutfitter_Settings.Outfits[categoryKey]

		for outfitKey, outfit in pairs(category) do
			local outfitName = outfit.Name;
			
			for itemKey, items in pairs(outfit["Items"]) do
				-- now we've got each item per set, lets cache them on an item level
				local itemid = items["Code"]
				
				if not Baggins_Outfitter.items_table[itemid] then
					Baggins_Outfitter.items_table[itemid] = {}
				end
				
				if itemid > 0 then
					table.insert(Baggins_Outfitter.items_table[itemid], outfitName)
				end -- itemid > 0
				
			end

		end

	end

end

local function OutfitUpdate()
	Baggins_Outfitter:CacheSets()
	Baggins:ForceFullUpdate()
end

function Baggins_Outfitter:OnEnable()
	self:CacheSets()

	Baggins:AddCustomRule("Outfitter", {
		DisplayName = "Outfitter",
		Description = "Filter by used in Outfitter set.",
		Matches = Matches,
		GetName = GetName,
		CleanRule = CleanRule,
		DewDropOptions = DewDropOptions
	})
	
	Outfitter:RegisterOutfitEvent("EDIT_OUTFIT", function () OutfitUpdate() end )
end


